@@ -1,4 +1,6 @@
 """Output formatters."""
+from __future__ import annotations
+
 import hashlib
 import json
 import os
@@ -41,8 +43,7 @@ class BaseFormatter(Generic[T]):
         # Use os.path.relpath 'cause Path.relative_to() misbehaves
         return os.path.relpath(path, start=self._base_dir)
 
-    # pylint: disable=no-self-use
-    def format(self, match: "MatchError") -> str:
+    def format(self, match: MatchError) -> str:
         """Format a match error."""
         return str(match)
 
@@ -55,7 +56,7 @@ class BaseFormatter(Generic[T]):
 class Formatter(BaseFormatter):  # type: ignore
     """Default output formatter of ansible-lint."""
 
-    def format(self, match: "MatchError") -> str:
+    def format(self, match: MatchError) -> str:
         _id = getattr(match.rule, "id", "000")
         result = f"[error_code]{_id}[/][dim]:[/] [error_title]{self.escape(match.message)}[/]"
         if match.tag:
@@ -73,7 +74,7 @@ class Formatter(BaseFormatter):  # type: ignore
 class QuietFormatter(BaseFormatter[Any]):
     """Brief output formatter for ansible-lint."""
 
-    def format(self, match: "MatchError") -> str:
+    def format(self, match: MatchError) -> str:
         return (
             f"[error_code]{match.rule.id}[/] "
             f"[filename]{self._format_path(match.filename or '')}[/]:{match.position}"
@@ -83,7 +84,7 @@ class QuietFormatter(BaseFormatter[Any]):
 class ParseableFormatter(BaseFormatter[Any]):
     """Parseable uses PEP8 compatible format."""
 
-    def format(self, match: "MatchError") -> str:
+    def format(self, match: MatchError) -> str:
         result = (
             f"[filename]{self._format_path(match.filename or '')}[/]:{match.position}: "
             f"[error_code]{match.rule.id}[/]"
@@ -112,7 +113,7 @@ class AnnotationsFormatter(BaseFormatter):  # type: ignore
     Supported levels: debug, warning, error
     """
 
-    def format(self, match: "MatchError") -> str:
+    def format(self, match: MatchError) -> str:
         """Prepare a match instance for reporting as a GitHub Actions annotation."""
         level = self._severity_to_level(match.rule.severity)
         file_path = self._format_path(match.filename or "")
@@ -147,7 +148,7 @@ class CodeclimateJSONFormatter(BaseFormatter[Any]):
     https://github.com/codeclimate/platform/blob/master/spec/analyzers/SPEC.md#user-content-data-types
     """
 
-    def format_result(self, matches: List["MatchError"]) -> str:
+    def format_result(self, matches: List[MatchError]) -> str:
         """Format a list of match errors as a JSON string."""
         if not isinstance(matches, list):
             raise RuntimeError(
@@ -158,10 +159,10 @@ class CodeclimateJSONFormatter(BaseFormatter[Any]):
         for match in matches:
             issue: Dict[str, Any] = {}
             issue["type"] = "issue"
-            issue["check_name"] = f"[{match.rule.id}] {match.message}"
+            issue["check_name"] = match.tag or match.rule.id  # rule-id[subrule-id]
             issue["categories"] = match.rule.tags
             issue["severity"] = self._severity_to_level(match.rule.severity)
-            issue["description"] = self.escape(str(match.rule.description))
+            issue["description"] = self.escape(str(match.message))
             issue["fingerprint"] = hashlib.sha256(
                 repr(match).encode("utf-8")
             ).hexdigest()
@@ -212,7 +213,7 @@ class SarifFormatter(BaseFormatter[Any]):
         "https://schemastore.azurewebsites.net/schemas/json/sarif-2.1.0-rtm.5.json"
     )
 
-    def format_result(self, matches: List["MatchError"]) -> str:
+    def format_result(self, matches: List[MatchError]) -> str:
         """Format a list of match errors as a JSON string."""
         if not isinstance(matches, list):
             raise RuntimeError(
@@ -254,7 +255,7 @@ class SarifFormatter(BaseFormatter[Any]):
         )
 
     def _extract_results(
-        self, matches: List["MatchError"]
+        self, matches: List[MatchError]
     ) -> Tuple[List[Any], List[Any]]:
         rules = {}
         results = []
@@ -264,7 +265,7 @@ class SarifFormatter(BaseFormatter[Any]):
             results.append(self._to_sarif_result(match))
         return list(rules.values()), results
 
-    def _to_sarif_rule(self, match: "MatchError") -> Dict[str, Any]:
+    def _to_sarif_rule(self, match: MatchError) -> Dict[str, Any]:
         rule: Dict[str, Any] = {
             "id": match.rule.id,
             "name": match.rule.id,
@@ -284,7 +285,7 @@ class SarifFormatter(BaseFormatter[Any]):
             rule["helpUri"] = match.rule.link
         return rule
 
-    def _to_sarif_result(self, match: "MatchError") -> Dict[str, Any]:
+    def _to_sarif_result(self, match: MatchError) -> Dict[str, Any]:
         result: Dict[str, Any] = {
             "ruleId": match.rule.id,
             "message": {
