@@ -10,7 +10,20 @@ from argparse import Namespace
 from collections import defaultdict
 from functools import lru_cache
 from importlib.abc import Loader
-from typing import Any, Dict, Iterator, List, Optional, Set, Union
+from typing import (
+    Any,
+    Dict,
+    Iterator,
+    List,
+    MutableMapping,
+    MutableSequence,
+    Optional,
+    Set,
+    Union,
+    cast,
+)
+
+from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
 import ansiblelint.skip_utils
 import ansiblelint.utils
@@ -21,7 +34,7 @@ from ansiblelint._internal.rules import (
     LoadingFailureRule,
     RuntimeErrorRule,
 )
-from ansiblelint.config import get_rule_config
+from ansiblelint.config import PROFILES, get_rule_config
 from ansiblelint.config import options as default_options
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, expand_paths_vars
@@ -47,15 +60,11 @@ class AnsibleLintRule(BaseRule):
         """Retrieve rule specific configuration."""
         return get_rule_config(self.id)
 
-    @lru_cache()
+    @lru_cache(maxsize=256)
     def get_config(self, key: str) -> Any:
         """Return a configured value for given key string."""
         return self.rule_config.get(key, None)
 
-    def __repr__(self) -> str:
-        """Return a AnsibleLintRule instance representation."""
-        return self.id + ": " + self.shortdesc
-
     @staticmethod
     def unjinja(text: str) -> str:
         """Remove jinja2 bits from a string."""
@@ -96,7 +105,17 @@ class AnsibleLintRule(BaseRule):
             frame = frame.f_back  # get the parent frame for the next iteration
         return match
 
-    def matchlines(self, file: "Lintable") -> List[MatchError]:
+    @staticmethod
+    def _enrich_matcherror_with_task_details(
+        match: MatchError, task: Dict[str, Any]
+    ) -> None:
+        match.task = task
+        if not match.details:
+            match.details = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
+        if match.linenumber < task[ansiblelint.utils.LINE_NUMBER_KEY]:
+            match.linenumber = task[ansiblelint.utils.LINE_NUMBER_KEY]
+
+    def matchlines(self, file: Lintable) -> List[MatchError]:
         matches: List[MatchError] = []
         if not self.match:
             return matches
@@ -134,34 +153,38 @@ class AnsibleLintRule(BaseRule):
         ):
             return matches
 
-        tasks_iterator = ansiblelint.yaml_utils.iter_tasks_in_file(file, self.id)
-        for raw_task, task, skipped, error in tasks_iterator:
+        tasks_iterator = ansiblelint.yaml_utils.iter_tasks_in_file(file)
+        for raw_task, task, skipped_tags, error in tasks_iterator:
             if error is not None:
                 # normalize_task converts AnsibleParserError to MatchError
                 return [error]
 
-            if skipped or "action" not in task:
+            if self.id in skipped_tags or ("action" not in task):
                 continue
 
             if self.needs_raw_task:
                 task["__raw_task__"] = raw_task
 
             result = self.matchtask(task, file=file)
-
             if not result:
                 continue
 
-            message = None
-            if isinstance(result, str):
-                message = result
-            task_msg = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
-            match = self.create_matcherror(
-                message=message,
-                linenumber=task[ansiblelint.utils.LINE_NUMBER_KEY],
-                details=task_msg,
-                filename=file,
-            )
-            match.task = task
+            if isinstance(result, MatchError):
+                if result.tag in skipped_tags:
+                    continue
+                match = result
+            else:
+                message = None
+                if isinstance(result, str):
+                    message = result
+                match = self.create_matcherror(
+                    message=message,
+                    linenumber=task[ansiblelint.utils.LINE_NUMBER_KEY],
+                    filename=file,
+                )
+
+            self._enrich_matcherror_with_task_details(match, task)
+
             matches.append(match)
         return matches
 
@@ -170,7 +193,7 @@ class AnsibleLintRule(BaseRule):
         if not self.matchplay or str(file.base_kind) != "text/yaml":
             return matches
 
-        yaml = ansiblelint.utils.parse_yaml_linenumbers(file)
+        yaml = file.data
         # yaml returned can be an AnsibleUnicode (a string) when the yaml
         # file contains a single string. YAML spec allows this but we consider
         # this an fatal error.
@@ -184,8 +207,6 @@ class AnsibleLintRule(BaseRule):
         if isinstance(yaml, dict):
             yaml = [yaml]
 
-        yaml = ansiblelint.skip_utils.append_skipped_rules(yaml, file)
-
         for play in yaml:
 
             # Bug #849
@@ -200,24 +221,115 @@ class AnsibleLintRule(BaseRule):
         return matches
 
 
+class TransformMixin:
+    """A mixin for AnsibleLintRule to enable transforming files.
+
+    If ansible-lint is started with the ``--write`` option, then the ``Transformer``
+    will call the ``transform()`` method for every MatchError identified if the rule
+    that identified it subclasses this ``TransformMixin``. Only the rule that identified
+    a MatchError can do transforms to fix that match.
+    """
+
+    def transform(
+        self,
+        match: MatchError,
+        lintable: Lintable,
+        data: Union[CommentedMap, CommentedSeq, str],
+    ) -> None:
+        """Transform ``data`` to try to fix the MatchError identified by this rule.
+
+        The ``match`` was generated by this rule in the ``lintable`` file.
+        When ``transform()`` is called on a rule, the rule should either fix the
+        issue, if possible, or make modifications that make it easier to fix manually.
+
+        The transform must set ``match.fixed = True`` when data has been transformed to
+        fix the error.
+
+        For YAML files, ``data`` is an editable YAML dict/array that preserves
+        any comments that were in the original file.
+
+        .. code:: python
+
+            data[0]["tasks"][0]["when"] = False
+
+        This is easier with the ``seek()`` utility method:
+
+        .. code :: python
+
+            target_task = self.seek(match.yaml_path, data)
+            target_task["when"] = False
+
+        For any files that aren't YAML, ``data`` is the loaded file's content as a string.
+        To edit non-YAML files, save the updated contents in ``lintable.content``:
+
+        .. code:: python
+
+            new_data = self.do_something_to_fix_the_match(data)
+            lintable.content = new_data
+        """
+
+    @staticmethod
+    def seek(
+        yaml_path: List[Union[int, str]],
+        data: Union[MutableMapping[str, Any], MutableSequence[Any], str],
+    ) -> Any:
+        """Get the element identified by ``yaml_path`` in ``data``.
+
+        Rules that work with YAML need to seek, or descend, into nested YAML data
+        structures to perform the relevant transforms. For example:
+
+        .. code:: python
+
+            def transform(self, match, lintable, data):
+                target_task = self.seek(match.yaml_path, data)
+                # transform target_task
+        """
+        if isinstance(data, str):
+            # can't descend into a string
+            return data
+        target = data
+        for segment in yaml_path:
+            # The cast() calls tell mypy what types we expect.
+            # Essentially this does:
+            #   target = target[segment]
+            if isinstance(segment, str):
+                target = cast(MutableMapping[str, Any], target)[segment]
+            elif isinstance(segment, int):
+                target = cast(MutableSequence[Any], target)[segment]
+        return target
+
+
 def is_valid_rule(rule: Any) -> bool:
     """Check if given rule is valid or not."""
     return issubclass(rule, AnsibleLintRule) and bool(rule.id) and bool(rule.shortdesc)
 
 
+# pylint: disable=too-many-nested-blocks
 def load_plugins(directory: str) -> Iterator[AnsibleLintRule]:
     """Yield a rule class."""
     for pluginfile in glob.glob(os.path.join(directory, "[A-Za-z]*.py")):
 
         pluginname = os.path.basename(pluginfile.replace(".py", ""))
         spec = importlib.util.spec_from_file_location(pluginname, pluginfile)
+
         # https://github.com/python/typeshed/issues/2793
         if spec and isinstance(spec.loader, Loader):
+
+            # load rule markdown documentation if found
+            help_md = ""
+            if spec.origin:
+                md_filename = os.path.splitext(spec.origin)[0] + ".md"
+                if os.path.exists(md_filename):
+                    with open(md_filename, encoding="utf-8") as f:
+                        help_md = f.read()
+
             module = importlib.util.module_from_spec(spec)
             spec.loader.exec_module(module)
             try:
                 for _, obj in inspect.getmembers(module):
                     if inspect.isclass(obj) and is_valid_rule(obj):
+                        if help_md:
+                            obj.help = help_md
                         yield obj()
 
             except (TypeError, ValueError, AttributeError) as exc:
@@ -253,13 +365,25 @@ class RulesCollection:
 
     def register(self, obj: AnsibleLintRule) -> None:
         """Register a rule."""
-        # We skip opt-in rules which were not manually enabled
-        if "opt-in" not in obj.tags or obj.id in self.options.enable_list:
+        # We skip opt-in rules which were not manually enabled.
+        # But we do include opt-in rules when listing all rules or tags
+        if any(
+            [
+                "opt-in" not in obj.tags,
+                obj.id in self.options.enable_list,
+                self.options.listrules,
+                self.options.listtags,
+            ]
+        ):
             self.rules.append(obj)
 
     def __iter__(self) -> Iterator[BaseRule]:
         """Return the iterator over the rules in the RulesCollection."""
-        return iter(self.rules)
+        return iter(sorted(self.rules))
+
+    def alphabetical(self) -> Iterator[BaseRule]:
+        """Return an iterator over the rules in the RulesCollection in alphabetical order."""
+        return iter(sorted(self.rules, key=lambda x: x.id))
 
     def __len__(self) -> int:
         """Return the length of the RulesCollection data."""
@@ -269,7 +393,7 @@ class RulesCollection:
         """Combine rules."""
         self.rules.extend(more)
 
-    def run(
+    def run(  # noqa: max-complexity: 12
         self,
         file: Lintable,
         tags: Optional[Set[str]] = None,
@@ -286,17 +410,19 @@ class RulesCollection:
             try:
                 if file.content is not None:  # loads the file content
                     pass
-            except IOError as exc:
+            except (IOError, UnicodeDecodeError) as exc:
                 return [
                     MatchError(
                         message=str(exc),
                         filename=file,
                         rule=LoadingFailureRule(),
-                        tag=exc.__class__.__name__.lower(),
+                        tag=f"{LoadingFailureRule.id}[{exc.__class__.__name__.lower()}]",
                     )
                 ]
 
         for rule in self.rules:
+            if rule.id == "syntax-check":
+                continue
             if (
                 not tags
                 or rule.has_dynamic_tags
@@ -352,3 +478,21 @@ class RulesCollection:
             for name in tags[tag]:
                 result += f"  - {name}\n"
         return result
+
+
+def filter_rules_with_profile(rule_col: RulesCollection, profile: str) -> None:
+    """Unload rules that are not part of the specified profile."""
+    included = set()
+    extends = profile
+    while extends:
+        for rule in PROFILES[extends]["rules"]:
+            included.add(rule)
+        extends = PROFILES[extends].get("extends", None)
+    for rule in list(rule_col.rules):
+        if rule.id not in included:
+            _logger.debug(
+                "Unloading %s rule due to not being part of %s profile.",
+                rule.id,
+                profile,
+            )
+            rule_col.rules.remove(rule)
