@@ -1,4 +1,6 @@
 """Implementation of yaml linting rule (yamllint integration)."""
+from __future__ import annotations
+
 import logging
 import sys
 from typing import TYPE_CHECKING, List
@@ -26,16 +28,18 @@ class YamllintRule(AnsibleLintRule):
     config = load_yamllint_config()
     has_dynamic_tags = True
     link = "https://yamllint.readthedocs.io/en/stable/rules.html"
+    # ensure this rule runs before most of other common rules
+    _order = 1
 
     def __init__(self) -> None:
         """Construct a rule instance."""
         # customize id by adding the one reported by yamllint
         self.id = self.__class__.id
 
-    def matchyaml(self, file: Lintable) -> List["MatchError"]:
+    def matchyaml(self, file: Lintable) -> List[MatchError]:
         """Return matches found for a specific YAML text."""
-        matches: List["MatchError"] = []
-        filtered_matches: List["MatchError"] = []
+        matches: List[MatchError] = []
+        filtered_matches: List[MatchError] = []
         if str(file.base_kind) != "text/yaml":
             return matches
 
@@ -94,10 +98,16 @@ if "pytest" in sys.modules:
                 "yaml",
                 [],
             ),
+            (
+                "examples/yamllint/multi-document.yaml",
+                "yaml",
+                [],
+            ),
         ),
         ids=(
             "invalid",
             "valid",
+            "multi-document",
         ),
     )
     def test_yamllint(file: str, expected_kind: str, expected: List[str]) -> None:
