@@ -19,23 +19,27 @@
 # THE SOFTWARE.
 
 """Utils related to inline skipping of rules."""
+from __future__ import annotations
+
+import collections.abc
 import logging
 from functools import lru_cache
 from itertools import product
-from typing import TYPE_CHECKING, Any, Generator, List, Optional, Sequence
+from typing import TYPE_CHECKING, Any, Dict, Generator, List, Optional, Sequence, Set
 
 # Module 'ruamel.yaml' does not explicitly export attribute 'YAML'; implicit reexport disabled
-from ruamel.yaml import YAML  # type: ignore
+from ruamel.yaml import YAML
+from ruamel.yaml.composer import ComposerError
 
 from ansiblelint.config import used_old_tags
-from ansiblelint.constants import RENAMED_TAGS
+from ansiblelint.constants import NESTED_TASK_KEYS, PLAYBOOK_TASK_KEYWORDS, RENAMED_TAGS
 from ansiblelint.file_utils import Lintable
 
 if TYPE_CHECKING:
     from ansible.parsing.yaml.objects import AnsibleBaseYAMLObject
 
 _logger = logging.getLogger(__name__)
-
+_found_deprecated_tags: Set[str] = set()
 
 # playbook: Sequence currently expects only instances of one of the two
 # classes below but we should consider avoiding this chimera.
@@ -46,13 +50,26 @@ _logger = logging.getLogger(__name__)
 def get_rule_skips_from_line(line: str) -> List[str]:
     """Return list of rule ids skipped via comment on the line of yaml."""
     _before_noqa, _noqa_marker, noqa_text = line.partition("# noqa")
-    noqa_text = noqa_text.lstrip(" :")
-    return noqa_text.split()
+
+    result = []
+    for v in noqa_text.lstrip(" :").split():
+        if v in RENAMED_TAGS:
+            tag = RENAMED_TAGS[v]
+            if v not in _found_deprecated_tags:
+                _logger.warning(
+                    "Replaced outdated tag '%s' with '%s', replace it to avoid future regressions",
+                    v,
+                    tag,
+                )
+                _found_deprecated_tags.add(v)
+            v = tag
+        result.append(v)
+    return result
 
 
 def append_skipped_rules(
-    pyyaml_data: "AnsibleBaseYAMLObject", lintable: Lintable
-) -> "AnsibleBaseYAMLObject":
+    pyyaml_data: AnsibleBaseYAMLObject, lintable: Lintable
+) -> AnsibleBaseYAMLObject:
     """Append 'skipped_rules' to individual tasks or single metadata block.
 
     For a file, uses 2nd parser (ruamel.yaml) to pull comments out of
@@ -78,7 +95,7 @@ def append_skipped_rules(
     return yaml_skip
 
 
-@lru_cache(maxsize=128)
+@lru_cache(maxsize=None)
 def load_data(file_text: str) -> Any:
     """Parse ``file_text`` as yaml and return parsed structure.
 
@@ -88,17 +105,34 @@ def load_data(file_text: str) -> Any:
     :return: Parsed yaml
     """
     yaml = YAML()
-    return yaml.load(file_text)
+    # Ruamel role is not to validate the yaml file, so we ignore duplicate keys:
+    yaml.allow_duplicate_keys = True
+    try:
+        return yaml.load(file_text)
+    except ComposerError:
+        # load fails on multi-documents with ComposerError exception
+        return yaml.load_all(file_text)
 
 
-def _append_skipped_rules(
-    pyyaml_data: "AnsibleBaseYAMLObject", lintable: Lintable
-) -> Optional["AnsibleBaseYAMLObject"]:
+def _append_skipped_rules(  # noqa: max-complexity: 12
+    pyyaml_data: AnsibleBaseYAMLObject, lintable: Lintable
+) -> Optional[AnsibleBaseYAMLObject]:
     # parse file text using 2nd parser library
     ruamel_data = load_data(lintable.content)
+    skipped_rules = _get_rule_skips_from_yaml(ruamel_data)
+
+    if lintable.kind in ["yaml", "requirements", "vars", "meta", "reno", "test-meta"]:
+        # AnsibleMapping, dict
+        if hasattr(pyyaml_data, "get"):
+            pyyaml_data["skipped_rules"] = skipped_rules
+        # AnsibleSequence, list
+        elif (
+            not isinstance(pyyaml_data, str)
+            and isinstance(pyyaml_data, collections.abc.Sequence)
+            and skipped_rules
+        ):
+            pyyaml_data[0]["skipped_rules"] = skipped_rules
 
-    if lintable.kind in ["yaml", "requirements", "vars", "meta", "reno"]:
-        pyyaml_data[0]["skipped_rules"] = _get_rule_skips_from_yaml(ruamel_data)
         return pyyaml_data
 
     # create list of blocks of tasks or nested tasks
@@ -110,9 +144,6 @@ def _append_skipped_rules(
             pyyaml_task_blocks = _get_task_blocks_from_playbook(pyyaml_data)
             ruamel_task_blocks = _get_task_blocks_from_playbook(ruamel_data)
         except (AttributeError, TypeError):
-            # TODO(awcrosby): running ansible-lint on any .yml file will
-            # assume it is a playbook, check needs to be added higher in the
-            # call stack, and can remove this except
             return pyyaml_data
     else:
         # For unsupported file types, we return empty skip lists
@@ -129,6 +160,10 @@ def _append_skipped_rules(
         if not pyyaml_task and not ruamel_task:
             continue
 
+        # AnsibleUnicode or str
+        if isinstance(pyyaml_task, str):
+            continue
+
         if pyyaml_task.get("name") != ruamel_task.get("name"):
             raise RuntimeError("Error in matching skip comment to a task")
         pyyaml_task["skipped_rules"] = _get_rule_skips_from_yaml(ruamel_task)
@@ -142,13 +177,6 @@ def _get_task_blocks_from_playbook(playbook: Sequence[Any]) -> List[Any]:
     :param playbook: playbook yaml from yaml parser.
     :returns: list of task dictionaries.
     """
-    PLAYBOOK_TASK_KEYWORDS = [
-        "tasks",
-        "pre_tasks",
-        "post_tasks",
-        "handlers",
-    ]
-
     task_blocks = []
     for play, key in product(playbook, PLAYBOOK_TASK_KEYWORDS):
         task_blocks.extend(play.get(key, []))
@@ -157,16 +185,18 @@ def _get_task_blocks_from_playbook(playbook: Sequence[Any]) -> List[Any]:
 
 def _get_tasks_from_blocks(task_blocks: Sequence[Any]) -> Generator[Any, None, None]:
     """Get list of tasks from list made of tasks and nested tasks."""
-    NESTED_TASK_KEYS = [
-        "block",
-        "always",
-        "rescue",
-    ]
+    if not task_blocks:
+        return
 
     def get_nested_tasks(task: Any) -> Generator[Any, None, None]:
+        if not task or not is_nested_task(task):
+            return
         for k in NESTED_TASK_KEYS:
-            if task and k in task and task[k]:
+            if k in task and task[k]:
+                if hasattr(task[k], "get"):
+                    continue
                 for subtask in task[k]:
+                    yield from get_nested_tasks(subtask)
                     yield subtask
 
     for task in task_blocks:
@@ -175,27 +205,33 @@ def _get_tasks_from_blocks(task_blocks: Sequence[Any]) -> Generator[Any, None, N
         yield task
 
 
-def _get_rule_skips_from_yaml(yaml_input: Sequence[Any]) -> Sequence[Any]:
+def _get_rule_skips_from_yaml(  # noqa: max-complexity: 12
+    yaml_input: Sequence[Any],
+) -> Sequence[Any]:
     """Traverse yaml for comments with rule skips and return list of rules."""
-    yaml_comment_obj_strs = []
+    yaml_comment_obj_strings = []
+
+    if isinstance(yaml_input, str):
+        return []
 
     def traverse_yaml(obj: Any) -> None:
-        yaml_comment_obj_strs.append(str(obj.ca.items))
+        yaml_comment_obj_strings.append(str(obj.ca.items))
         if isinstance(obj, dict):
             for _, val in obj.items():
                 if isinstance(val, (dict, list)):
                     traverse_yaml(val)
         elif isinstance(obj, list):
-            for e in obj:
-                if isinstance(e, (dict, list)):
-                    traverse_yaml(e)
+            for element in obj:
+                if isinstance(element, (dict, list)):
+                    traverse_yaml(element)
         else:
             return
 
-    traverse_yaml(yaml_input)
+    if isinstance(yaml_input, (dict, list)):
+        traverse_yaml(yaml_input)
 
     rule_id_list = []
-    for comment_obj_str in yaml_comment_obj_strs:
+    for comment_obj_str in yaml_comment_obj_strings:
         for line in comment_obj_str.split(r"\n"):
             rule_id_list.extend(get_rule_skips_from_line(line))
 
@@ -208,3 +244,16 @@ def normalize_tag(tag: str) -> str:
         used_old_tags[tag] = RENAMED_TAGS[tag]
         return RENAMED_TAGS[tag]
     return tag
+
+
+def is_nested_task(task: Dict[str, Any]) -> bool:
+    """Check if task includes block/always/rescue."""
+    # Cannot really trust the input
+    if isinstance(task, str):
+        return False
+
+    for key in NESTED_TASK_KEYS:
+        if task.get(key):
+            return True
+
+    return False
