@@ -1,4 +1,5 @@
 """Test utils for ansible-lint."""
+from __future__ import annotations
 
 import os
 import shutil
@@ -14,6 +15,7 @@ if TYPE_CHECKING:
     # https://github.com/PyCQA/pylint/issues/3240
     # pylint: disable=unsubscriptable-object
     CompletedProcess = subprocess.CompletedProcess[Any]
+    from ansiblelint.errors import MatchError  # noqa: E402
 else:
     CompletedProcess = subprocess.CompletedProcess
 
@@ -22,7 +24,6 @@ else:
 app = get_app(offline=True)
 
 # pylint: disable=wrong-import-position
-from ansiblelint.errors import MatchError  # noqa: E402
 from ansiblelint.runner import Runner  # noqa: E402
 
 
@@ -33,7 +34,7 @@ class RunFromText:
         """Initialize a RunFromText instance with rules collection."""
         self.collection = collection
 
-    def _call_runner(self, path: str) -> List["MatchError"]:
+    def _call_runner(self, path: str) -> List[MatchError]:
         runner = Runner(path, rules=self.collection)
         return runner.run()
 
@@ -85,7 +86,7 @@ def run_ansible_lint(
     *argv: str,
     cwd: Optional[str] = None,
     executable: Optional[str] = None,
-    env: Optional[Dict[str, str]] = None
+    env: Optional[Dict[str, str]] = None,
 ) -> CompletedProcess:
     """Run ansible-lint on a given path and returns its output."""
     if not executable:
